<?php

namespace Drupal\snippet_manager\Form;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Entity\EntityForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a base class for variable forms.
 *
 * @property \Drupal\snippet_manager\SnippetInterface $entity
 */
abstract class VariableFormBase extends EntityForm {

  /**
   * The variable plugin.
   *
   * @var \Drupal\snippet_manager\SnippetVariableInterface
   */
  protected $plugin;

  /**
   * The variable manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $variableManager;

  /**
   * Constructs form object object.
   */
  public function __construct(PluginManagerInterface $variable_manager) {
    $this->variableManager = $variable_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.snippet_variable')
    );
  }

  /**
   * Plugin getter.
   *
   * The plugin cannot be set in the constructor because form $entity is not
   * ready yet at that moment.
   */
  protected function getPlugin() {
    if (!$this->plugin) {
      $variable = $this->entity->getVariable($this->getVariableName());
      $this->plugin = $this->variableManager->createInstance(
        $variable['plugin_id'],
        $variable['configuration']
      );
    }
    return $this->plugin;
  }

  /**
   * Returns name of the variable.
   */
  protected function getVariableName() {
    return $this->getRouteMatch()->getParameter('variable');
  }

}
